@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Oauth2client tests.
+"""oauth2client tests.
 
 Unit tests for service account credentials implemented using RSA.
 """
@@ -21,17 +21,18 @@ import datetime
 import json
 import os
 import tempfile
+import unittest
 
-import httplib2
 import mock
 import rsa
-from six import BytesIO
-import unittest2
+import six
+from six.moves import http_client
 
 from oauth2client import client
 from oauth2client import crypt
 from oauth2client import service_account
-from .http_mock import HttpMockSequence
+from oauth2client import transport
+from tests import http_mock
 
 
 def data_filename(filename):
@@ -43,9 +44,11 @@ def datafile(filename):
         return file_obj.read()
 
 
-class ServiceAccountCredentialsTests(unittest2.TestCase):
+class ServiceAccountCredentialsTests(unittest.TestCase):
 
     def setUp(self):
+        self.orig_signer = crypt.Signer
+        self.orig_verifier = crypt.Verifier
         self.client_id = '123'
         self.service_account_email = 'dummy@google.com'
         self.private_key_id = 'ABCDEF'
@@ -59,6 +62,10 @@ class ServiceAccountCredentialsTests(unittest2.TestCase):
             client_id=self.client_id,
         )
 
+    def tearDown(self):
+        crypt.Signer = self.orig_signer
+        crypt.Verifier = self.orig_verifier
+
     def test__to_json_override(self):
         signer = object()
         creds = service_account.ServiceAccountCredentials(
@@ -175,7 +182,7 @@ class ServiceAccountCredentialsTests(unittest2.TestCase):
                 scopes=scopes, token_uri=token_uri, revoke_uri=revoke_uri))
         creds_from_file_contents = (
             service_account.ServiceAccountCredentials.from_p12_keyfile_buffer(
-                service_account_email, BytesIO(key_contents),
+                service_account_email, six.BytesIO(key_contents),
                 private_key_password=private_key_password,
                 scopes=scopes, token_uri=token_uri, revoke_uri=revoke_uri))
         for creds in (creds_from_filename, creds_from_file_contents):
@@ -270,10 +277,10 @@ class ServiceAccountCredentialsTests(unittest2.TestCase):
         utcnow.return_value = NOW
 
         # Create a custom credentials with a mock signer.
-        signer = mock.MagicMock()
+        signer = mock.Mock()
         signed_value = b'signed-content'
-        signer.sign = mock.MagicMock(name='sign',
-                                     return_value=signed_value)
+        signer.sign = mock.Mock(name='sign',
+                                return_value=signed_value)
         credentials = service_account.ServiceAccountCredentials(
             self.service_account_email,
             signer,
@@ -296,10 +303,10 @@ class ServiceAccountCredentialsTests(unittest2.TestCase):
             'access_token': token2,
             'expires_in': lifetime,
         }
-        http = HttpMockSequence([
-            ({'status': '200'},
+        http = http_mock.HttpMockSequence([
+            ({'status': http_client.OK},
              json.dumps(token_response_first).encode('utf-8')),
-            ({'status': '200'},
+            ({'status': http_client.OK},
              json.dumps(token_response_second).encode('utf-8')),
         ])
 
@@ -379,7 +386,7 @@ T3_EXPIRY = T3 + TOKEN_LIFE
 T3_EXPIRY_DATE = T3_DATE + datetime.timedelta(seconds=TOKEN_LIFE)
 
 
-class JWTAccessCredentialsTests(unittest2.TestCase):
+class JWTAccessCredentialsTests(unittest.TestCase):
 
     def setUp(self):
         self.client_id = '123'
@@ -400,13 +407,15 @@ class JWTAccessCredentialsTests(unittest2.TestCase):
         time.return_value = T1
 
         token_info = self.jwt.get_access_token()
+        certs = {'key': datafile('public_cert.pem')}
         payload = crypt.verify_signed_jwt_with_certs(
-            token_info.access_token,
-            {'key': datafile('public_cert.pem')}, audience=self.url)
+            token_info.access_token, certs, audience=self.url)
+        self.assertEqual(len(payload), 5)
         self.assertEqual(payload['iss'], self.service_account_email)
         self.assertEqual(payload['sub'], self.service_account_email)
         self.assertEqual(payload['iat'], T1)
         self.assertEqual(payload['exp'], T1_EXPIRY)
+        self.assertEqual(payload['aud'], self.url)
         self.assertEqual(token_info.expires_in, T1_EXPIRY - T1)
 
         # Verify that we vend the same token after 100 seconds
@@ -437,19 +446,20 @@ class JWTAccessCredentialsTests(unittest2.TestCase):
         utcnow.return_value = T1_DATE
         time.return_value = T1
 
-        token_info = self.jwt.get_access_token(
-            additional_claims={'aud': 'https://test2.url.com',
-                               'sub': 'dummy2@google.com'
-                               })
+        audience = 'https://test2.url.com'
+        subject = 'dummy2@google.com'
+        claims = {'aud': audience, 'sub': subject}
+        token_info = self.jwt.get_access_token(additional_claims=claims)
+        certs = {'key': datafile('public_cert.pem')}
         payload = crypt.verify_signed_jwt_with_certs(
-            token_info.access_token,
-            {'key': datafile('public_cert.pem')},
-            audience='https://test2.url.com')
+            token_info.access_token, certs, audience=audience)
         expires_in = token_info.expires_in
+        self.assertEqual(len(payload), 5)
         self.assertEqual(payload['iss'], self.service_account_email)
-        self.assertEqual(payload['sub'], 'dummy2@google.com')
+        self.assertEqual(payload['sub'], subject)
         self.assertEqual(payload['iat'], T1)
         self.assertEqual(payload['exp'], T1_EXPIRY)
+        self.assertEqual(payload['aud'], audience)
         self.assertEqual(expires_in, T1_EXPIRY - T1)
 
     def test_revoke(self):
@@ -474,30 +484,36 @@ class JWTAccessCredentialsTests(unittest2.TestCase):
         utcnow.return_value = T1_DATE
         time.return_value = T1
 
-        def mock_request(uri, method='GET', body=None, headers=None,
-                         redirections=0, connection_type=None):
-            self.assertEqual(uri, self.url)
-            bearer, token = headers[b'Authorization'].split()
+        http = http_mock.HttpMockSequence([
+            ({'status': http_client.OK}, b''),
+            ({'status': http_client.OK}, b''),
+        ])
+
+        self.jwt.authorize(http)
+        transport.request(http, self.url)
+
+        # Ensure we use the cached token
+        utcnow.return_value = T2_DATE
+        transport.request(http, self.url)
+
+        # Verify mocks.
+        certs = {'key': datafile('public_cert.pem')}
+        self.assertEqual(len(http.requests), 2)
+        for info in http.requests:
+            self.assertEqual(info['method'], 'GET')
+            self.assertEqual(info['uri'], self.url)
+            self.assertIsNone(info['body'])
+            self.assertEqual(len(info['headers']), 1)
+            bearer, token = info['headers'][b'Authorization'].split()
+            self.assertEqual(bearer, b'Bearer')
             payload = crypt.verify_signed_jwt_with_certs(
-                token,
-                {'key': datafile('public_cert.pem')},
-                audience=self.url)
+                token, certs, audience=self.url)
+            self.assertEqual(len(payload), 5)
             self.assertEqual(payload['iss'], self.service_account_email)
             self.assertEqual(payload['sub'], self.service_account_email)
             self.assertEqual(payload['iat'], T1)
             self.assertEqual(payload['exp'], T1_EXPIRY)
-            self.assertEqual(uri, self.url)
-            self.assertEqual(bearer, b'Bearer')
-            return (httplib2.Response({'status': '200'}), b'')
-
-        h = httplib2.Http()
-        h.request = mock_request
-        self.jwt.authorize(h)
-        h.request(self.url)
-
-        # Ensure we use the cached token
-        utcnow.return_value = T2_DATE
-        h.request(self.url)
+            self.assertEqual(payload['aud'], self.url)
 
     @mock.patch('oauth2client.client._UTCNOW')
     @mock.patch('time.time')
@@ -509,65 +525,126 @@ class JWTAccessCredentialsTests(unittest2.TestCase):
             self.service_account_email, self.signer,
             private_key_id=self.private_key_id, client_id=self.client_id)
 
-        def mock_request(uri, method='GET', body=None, headers=None,
-                         redirections=0, connection_type=None):
-            self.assertEqual(uri, self.url)
-            bearer, token = headers[b'Authorization'].split()
-            payload = crypt.verify_signed_jwt_with_certs(
-                token,
-                {'key': datafile('public_cert.pem')},
-                audience=self.url)
-            self.assertEqual(payload['iss'], self.service_account_email)
-            self.assertEqual(payload['sub'], self.service_account_email)
-            self.assertEqual(payload['iat'], T1)
-            self.assertEqual(payload['exp'], T1_EXPIRY)
-            self.assertEqual(uri, self.url)
-            self.assertEqual(bearer, b'Bearer')
-            return httplib2.Response({'status': '200'}), b''
+        http = http_mock.HttpMockSequence([
+            ({'status': http_client.OK}, b''),
+        ])
 
-        h = httplib2.Http()
-        h.request = mock_request
-        jwt.authorize(h)
-        h.request(self.url)
+        jwt.authorize(http)
+        transport.request(http, self.url)
 
         # Ensure we do not cache the token
         self.assertIsNone(jwt.access_token)
 
+        # Verify mocks.
+        self.assertEqual(len(http.requests), 1)
+        info = http.requests[0]
+        self.assertEqual(info['method'], 'GET')
+        self.assertEqual(info['uri'], self.url)
+        self.assertIsNone(info['body'])
+        self.assertEqual(len(info['headers']), 1)
+        bearer, token = info['headers'][b'Authorization'].split()
+        self.assertEqual(bearer, b'Bearer')
+        certs = {'key': datafile('public_cert.pem')}
+        payload = crypt.verify_signed_jwt_with_certs(
+            token, certs, audience=self.url)
+        self.assertEqual(len(payload), 5)
+        self.assertEqual(payload['iss'], self.service_account_email)
+        self.assertEqual(payload['sub'], self.service_account_email)
+        self.assertEqual(payload['iat'], T1)
+        self.assertEqual(payload['exp'], T1_EXPIRY)
+        self.assertEqual(payload['aud'], self.url)
+
     @mock.patch('oauth2client.client._UTCNOW')
     def test_authorize_stale_token(self, utcnow):
         utcnow.return_value = T1_DATE
         # Create an initial token
-        h = HttpMockSequence([({'status': '200'}, b''),
-                              ({'status': '200'}, b'')])
-        self.jwt.authorize(h)
-        h.request(self.url)
+        http = http_mock.HttpMockSequence([
+            ({'status': http_client.OK}, b''),
+            ({'status': http_client.OK}, b''),
+        ])
+        self.jwt.authorize(http)
+        transport.request(http, self.url)
         token_1 = self.jwt.access_token
 
         # Expire the token
         utcnow.return_value = T3_DATE
-        h.request(self.url)
+        transport.request(http, self.url)
         token_2 = self.jwt.access_token
         self.assertEquals(self.jwt.token_expiry, T3_EXPIRY_DATE)
         self.assertNotEqual(token_1, token_2)
 
+        # Verify mocks.
+        certs = {'key': datafile('public_cert.pem')}
+        self.assertEqual(len(http.requests), 2)
+        issued_at_vals = (T1, T3)
+        exp_vals = (T1_EXPIRY, T3_EXPIRY)
+        for info, issued_at, exp_val in zip(http.requests, issued_at_vals,
+                                            exp_vals):
+            self.assertEqual(info['uri'], self.url)
+            self.assertEqual(info['method'], 'GET')
+            self.assertIsNone(info['body'])
+            self.assertEqual(len(info['headers']), 1)
+            bearer, token = info['headers'][b'Authorization'].split()
+            self.assertEqual(bearer, b'Bearer')
+            # To parse the token, skip the time check, since this
+            # test intentionally has stale tokens.
+            with mock.patch('oauth2client.crypt._verify_time_range',
+                            return_value=True):
+                payload = crypt.verify_signed_jwt_with_certs(
+                    token, certs, audience=self.url)
+            self.assertEqual(len(payload), 5)
+            self.assertEqual(payload['iss'], self.service_account_email)
+            self.assertEqual(payload['sub'], self.service_account_email)
+            self.assertEqual(payload['iat'], issued_at)
+            self.assertEqual(payload['exp'], exp_val)
+            self.assertEqual(payload['aud'], self.url)
+
     @mock.patch('oauth2client.client._UTCNOW')
     def test_authorize_401(self, utcnow):
         utcnow.return_value = T1_DATE
 
-        h = HttpMockSequence([
-            ({'status': '200'}, b''),
-            ({'status': '401'}, b''),
-            ({'status': '200'}, b'')])
-        self.jwt.authorize(h)
-        h.request(self.url)
+        http = http_mock.HttpMockSequence([
+            ({'status': http_client.OK}, b''),
+            ({'status': http_client.UNAUTHORIZED}, b''),
+            ({'status': http_client.OK}, b''),
+        ])
+        self.jwt.authorize(http)
+        transport.request(http, self.url)
         token_1 = self.jwt.access_token
 
         utcnow.return_value = T2_DATE
-        self.assertEquals(h.request(self.url)[0].status, 200)
+        response, _ = transport.request(http, self.url)
+        self.assertEquals(response.status, http_client.OK)
         token_2 = self.jwt.access_token
         # Check the 401 forced a new token
         self.assertNotEqual(token_1, token_2)
 
+        # Verify mocks.
+        certs = {'key': datafile('public_cert.pem')}
+        self.assertEqual(len(http.requests), 3)
+        issued_at_vals = (T1, T1, T2)
+        exp_vals = (T1_EXPIRY, T1_EXPIRY, T2_EXPIRY)
+        for info, issued_at, exp_val in zip(http.requests, issued_at_vals,
+                                            exp_vals):
+            self.assertEqual(info['uri'], self.url)
+            self.assertEqual(info['method'], 'GET')
+            self.assertIsNone(info['body'])
+            self.assertEqual(len(info['headers']), 1)
+            bearer, token = info['headers'][b'Authorization'].split()
+            self.assertEqual(bearer, b'Bearer')
+            # To parse the token, skip the time check, since this
+            # test intentionally has stale tokens.
+            with mock.patch('oauth2client.crypt._verify_time_range',
+                            return_value=True):
+                payload = crypt.verify_signed_jwt_with_certs(
+                    token, certs, audience=self.url)
+            self.assertEqual(len(payload), 5)
+            self.assertEqual(payload['iss'], self.service_account_email)
+            self.assertEqual(payload['sub'], self.service_account_email)
+            self.assertEqual(payload['iat'], issued_at)
+            self.assertEqual(payload['exp'], exp_val)
+            self.assertEqual(payload['aud'], self.url)
+
     @mock.patch('oauth2client.client._UTCNOW')
     def test_refresh(self, utcnow):
         utcnow.return_value = T1_DATE
